; script:MapFold.scm
; author: Tin Tran
; date: April 15, 2016
; Map Fold effect
; creates a mapfold effect out of exist image/layer like shown here: http://gimpchat.com/viewtopic.php?f=23&t=4439
; MrQ change menu location from: Script-Fu/Create New/Map Fold Effect... to: Filters/Map/Map Fold Effect...
(define (script-fu-map-fold image layer 
            Y-Shift-P
			
         )
	
		(let* 
		   ((Y-Shift 0)
		   (width (car (gimp-image-width image)))
		   (height (car (gimp-image-height image)))
		   (sheared-item 0)
		   (transformed-item 0)
		   (copied-layer 0)
		   )
			;(gimp-image-undo-disable image); DN = NO UNDO
			(set! Y-Shift (/ (* height Y-Shift-P) 100))
			(gimp-image-undo-group-start image)                   ;undo-group in one step
			
			;resize canvas to double the image size and center layer
			(gimp-image-resize image (* width 2) (* height 2) (/ width 2) (/ height 2))
			(gimp-layer-resize-to-image-size layer)
			
			;select and shear
			(gimp-rect-select image (/ width 2) (/ height 2) (ceiling (/ width 4)) height CHANNEL-OP-REPLACE FALSE 0)
			(gimp-item-transform-shear layer ORIENTATION-VERTICAL (- 0 Y-Shift))
			
			(gimp-rect-select image (+ (/ width 2) (ceiling (/ width 4))) (/ height 2) (ceiling (/ width 4)) height CHANNEL-OP-REPLACE FALSE 0)
			(gimp-item-transform-shear layer ORIENTATION-VERTICAL Y-Shift)
			
			(gimp-rect-select image (+ (/ width 2) (* (ceiling (/ width 4)) 2)) (/ height 2) (ceiling (/ width 4)) height CHANNEL-OP-REPLACE FALSE 0)
			(gimp-item-transform-shear layer ORIENTATION-VERTICAL (- 0 Y-Shift))
			
			(gimp-rect-select image (+ (/ width 2) (* (ceiling (/ width 4)) 3)) (/ height 2) (ceiling (/ width 4)) height CHANNEL-OP-REPLACE FALSE 0)
			(set! sheared-item (car (gimp-item-transform-shear layer ORIENTATION-VERTICAL Y-Shift)))
			
			;anchor the last sheared item to flaten image
			(gimp-floating-sel-anchor sheared-item)
			
			;select the even sections (section 2 and 4) to darken a bit
			(gimp-rect-select image (+ (/ width 2) (ceiling (/ width 4))) 0 (ceiling (/ width 4)) (* height 2) CHANNEL-OP-REPLACE FALSE 0)
			(gimp-rect-select image (+ (/ width 2) (* (ceiling (/ width 4)) 3)) 0 (ceiling (/ width 4)) (* height 2) CHANNEL-OP-ADD FALSE 0)
		
		    ;darken selected area a bit
			(gimp-brightness-contrast layer -50 0)
			
			;apply perspective
			(gimp-selection-all image)
			(set! transformed-item (car (gimp-item-transform-perspective layer (/ width 4) 0 (- (* width 2) (/ width 4)) 0
			                                        0 (* height 2) (* width 2) (* height 2))))
			(gimp-floating-sel-anchor transformed-item)									
												
		    ;copy the layer 
			(set! copied-layer (car (gimp-layer-new-from-drawable layer image)))
            (gimp-image-insert-layer image copied-layer 0 1)
			
			;set it to black
			(gimp-image-select-item image CHANNEL-OP-REPLACE copied-layer)
			(gimp-context-set-background '(0 0 0))
			(gimp-edit-fill copied-layer BACKGROUND-FILL)
			
			;apply perspective
			(gimp-selection-all image)
			(set! transformed-item (car (gimp-item-transform-perspective copied-layer 0 (/ height 2) (* width 2) (/ height 2)
			                                        0 (* height 2) (* width 2) (* height 2))))
			(gimp-floating-sel-anchor transformed-item)
			
			;blur this layer to make it look like shadow
			(gimp-selection-all image)
			(plug-in-gauss RUN-NONINTERACTIVE image copied-layer (/ width 20) (/ width 20) 1)
			
			;set opacity 50%
			(gimp-layer-set-opacity copied-layer 50)
			
			
		   ;(gimp-image-undo-enable image) ;DN = NO UNDO
			(gimp-image-undo-group-end image)                     ;undo group in one step
			(gimp-displays-flush)
	    )
	
	
    
) ;end of define

(script-fu-register
  "script-fu-map-fold"         ;function name
  "Map Fold Effect..."    ;menu register
  "Creates Map Fold effect"       ;description
  "Tin Tran"                          ;author name
  "copyright info and description"         ;copyright info or description
  "2016"                          ;date
  "RGB*, GRAY*"                        ;mode
  SF-IMAGE      "Image" 0                   
  SF-DRAWABLE   "Layer" 0
  SF-ADJUSTMENT "Shear Magnitude _Y Percentage (height %)" '(15 5 45 1 5 0 0)
)
(script-fu-menu-register "script-fu-map-fold"
                         "<Image>/Filters/Map")